whatif <- function(data, cfact, range = NULL, freq = NULL, nearby = NULL, 
 miss = "list", form = NULL, ...)  {

 #INSTALL AND LOAD REQUIRED PACKAGE lpSolve
 if (!("lpSolve" %in% .packages(all = TRUE))) {
   install.packages("lpSolve")
 }
 require(lpSolve)

 #DATA PROCESSING AND RELATED USER INPUT ERROR CHECKING
 #Initial user input error checking for cfact and data
 if(!(is.character(data) || is.data.frame(data) || is.matrix(data) || 
   class(data) == "z.out")) {
   stop("'data' must be either a string, a R data frame, a R matrix, or a Zelig output object")
 }

 if(!(is.character(cfact) || is.data.frame(cfact) || is.matrix(cfact))) {
   stop("'cfact' must be either a string, a R data frame, or a R matrix")
 }

 #Load cfact and data from external files if arguments data and cfact are strings
 if (is.character(data))  {
   data <- read.table(data)
 }

 if (is.character(cfact))  {
   cfact <- read.table(cfact)
 }

 #Extract observed covariate data if argument data is a Zelig output object
   if(class(data) == "z.out")  {
     tt <- terms(data)
     attr(tt, "intercept") <- rep(0, length(attr(tt, "intercept")))
     data <- model.matrix(tt, data = model.frame(tt))
     rm(tt)
   }  

 #Check that arguments cfact and data are not empty data frames or matrices
 if (dim(data)[1] == 0)  {
   stop("no factual data supplied: 'data' contains zero rows")
 }

 if (dim(cfact)[1] == 0)  {
   stop("no counterfactuals supplied: 'cfact' contains zero rows")
 }

 #Check if there are no cases without missing values in data and cfact
 if (!any(complete.cases(data)))  {
   stop("there are no cases in 'data' without missing values")
 }
 if (!any(complete.cases(cfact)))  {
   stop("there are no cases in 'cfact' without missing values")
 }

 #List-wise delete any cases with missing data in cfact
 if (!(identical(complete.cases(cfact), rep(TRUE, dim(cfact)[1])))) {
   cfact <- na.omit(cfact)
   warning("Counterfactuals with missing values eliminated from cfact")
 }

 #Use formula to modify data and cfact, after user input error checking
 if (!(is.null(form)))  {
   if (!(is.data.frame(data)))  {
     stop("'data' must be a data frame in order to use 'form'")
   }
   if (!(is.data.frame(cfact)))  {
     stop("'cfact' must be a data frame in order to use 'form'")
   }
   if (!(identical(class(form), "formula")))  {
     stop("'form' must be a formula with class 'formula'")
   }
   form <- update.formula(form, ~ . -1)
   data <- model.matrix(form, data = data)
   cfact <- model.matrix(form, data = cfact)
 }

 #Convert data frames to numeric matrices, coercing factors to internal numeric
 #representations
 if (is.data.frame(data))  {
   data <- suppressWarnings(data.matrix(data))
 }

 if (is.data.frame(cfact))  {
   cfact <- suppressWarnings(data.matrix(cfact))
 } 

 #Final check on key input data:  are both arguments data and cfact now numeric 
 #matrices after loading, initial manipulations, etc. and is cfact fully observed?  
 if (!(is.matrix(data) && is.numeric(data)))  {
   stop("'data' must either be a numeric matrix or a data frame coercable to one")
 }

 if (!(is.matrix(cfact) && is.numeric(cfact)))  {
   stop("'cfact' must be either a numeric matrix or a data frame coercable to one")
  }

 na.fail(cfact)

 #NON DATA-PROCESSING RELATED USER INPUT ERROR CHECKING
 #Check if cfact, data have the same number of dimensions, k
 if (!identical(ncol(cfact), ncol(data)))  {
   stop("number of columns of 'cfact' and 'data' are not equal")
 }

 #Check format of range if user supplies an argument
 if (!(is.null(range)))  {
   #Check if range is a numeric vector
   if (!(is.vector(range) && is.numeric(range)))  {
     stop("'range' must be a numeric vector")
   }

   #Check if length of range equals the number of columns of data, k
   if (!identical(length(range), ncol(data)))  {
     stop("length of 'range' does not equal number of columns of 'data'")
   }
 }

 #Check format of freq if user supplies an argument
 if (!(is.null(freq)))  {
   #Check if freq is a numeric vector
   if (!(is.vector(freq) && is.numeric(freq)))  {
     stop("'freq' must be a numeric vector")
   }

   #Check if there is missing data in freq
     na.fail(freq)
 }

 #Check if nearby argument is numeric, greater than or equal to 0, and a vector
 #of length 1, if supplied
 if (!(is.null(nearby))) {
   if (!(is.numeric(nearby) && nearby >= 0 && length(nearby) == 1))  {
     stop("'nearby' must be numeric, greater than or equal to 0, and a scalar")
   }
 }

 #Check if miss argument is valid
 if (!(is.character(miss) && (identical(miss, "list") || identical(miss, "case"))))  {
   stop("'miss' must be either ''case'' or ''list''")
 }

 #KEY LOCAL VARIABLES
 n = nrow(data)  #Number of data points in observed data set
 m = nrow(cfact)  #Number of (fully observed) counterfactuals
 k = ncol(data)  #Number of dimensions (covariates)

 #LOCAL FUNCTIONS
 convex.hull.test <- function(m, dat, cf)  {
   test.result <- FALSE
   #Create objects required by lp function, adding a row of 1s to transposed matrix 
   #dat and a 1 to vector cf[m,].  Note that "A" here corresponds to "A'" in King
   #and Zeng 2006a, Appendix A, and "B" and "C" to their "B" and "C", respectively.
   A <- rbind(t(dat), rep(1, nrow(dat)))
   B <- c(cf[m,],1)
   C <- c(rep(0, nrow(dat)))
   D <- c(rep("=", ncol(dat) + 1))
	
   #Use lp to implement test.
   lp.result <- lp(objective.in=C, const.mat=A, const.dir=D, const.rhs=B)
   if (lp.result$status == 0)  {
     test.result <- TRUE
   }
   return(test.result)
 }	

 calc.gd <- function(m, dat, cf, rang) {
   #If range =  0 for a variable k, set the normalized difference
   #equal to 0 if, for a given observed data point p, its
   #kth element minus the kth element of the counterfactual is 0. 
   #Otherwise set equal to NA, thus ignoring the contribution of the kth
   #variable to the calculation of Gower's distance.
   #Note that an element of the range vector should only be 0 in degenerate 
   #cases.
   if (identical(TRUE, any(rang == 0)))  {
     temp <- t(abs(t(dat) - cf[m,]))
     for (j in 1:length(rang))  {
       for (i in 1:nrow(dat))  {
	 if (identical(0, rang[j]))  {
	   if (!identical(0, temp[i, j]))  {
	     is.na(temp[i, j]) <- TRUE
	   }
	 }  else {
	   temp[i, j] <- temp[i, j]/rang[j]
	 }
	}
     }
     prelim.gd <- apply(temp, 1, mean, na.rm = TRUE)
   }  else  {
     prelim.gd <- apply(t(abs(t(dat) - cf[m,])/rang), 1, mean, na.rm = TRUE)

   }
   for (i in 1:length(prelim.gd))  {
     if (is.nan(prelim.gd[i]))  {
       is.na(prelim.gd[i]) <- TRUE
     }
   }
   return(prelim.gd)
 }

 calc.cumfreq <- function(i, freq, dist)  {
   colSums(dist <= freq[i])  
 }

 #MISSING DATA
 is.miss <- !identical(complete.cases(data), rep(TRUE, n))
 if (is.miss && identical(miss, "list"))  {
   data <- na.omit(data)
   n <- nrow(data)
 }

 #CONVEX HULL TEST
 #If the mth counterfactual is one of the points in data,
 #return that it is in the convex hull; if it is not one of the 
 #points in data, use linear programming to test if it is in
 #the convex hull.
 if (is.miss && identical(miss, "case"))  {
   test.result <- sapply(1:m, convex.hull.test,  
     dat = na.omit(data), cf = cfact)
 }  else  {
   test.result <- sapply(1:m, convex.hull.test, dat = data, cf = cfact)
 }

 #GOWER'S DISTANCE
 samp.range <- apply(data, 2, max, na.rm = TRUE) - apply(data, 2, min, na.rm = TRUE)
 if (identical(range, NULL))  {
   gowdist <- matrix(sapply(1:m, calc.gd, dat = data, cf = cfact,
     rang = samp.range), ncol = m)
 }  else  {
   for (i in 1:k)  {
     if (!is.na(range[i]))  {
       samp.range[i] <- range[i]
     }
   }
   gowdist <- matrix(sapply(1:m, calc.gd, dat = data, cf = cfact, rang = samp.range), 
     ncol = m)
 }
 dimnames(gowdist) <- list(seq(1, n, by = 1), seq(1, m, by = 1))

 #GEOMETRIC VARIANCE
 gd.x <- matrix(sapply(1:n, calc.gd, dat = data, cf = data,
   rang = samp.range), ncol = n)
 sum.gd.x <- sum(apply(gd.x, 1, sum, na.rm = TRUE))
 gv.x <- (0.5 * sum.gd.x)/(n^2)
	
 #SUMMARY STATISTIC
 if (identical(nearby, NULL))  {
   nearby <- gv.x
 }
 summary <- colSums(gowdist <= nearby) * (1/n)

 #CUMULATIVE FREQUENCIES
 if (identical(freq, NULL))  {
   freqdist <- seq(0, 1, by = 0.05)
 } else {
   freqdist <- freq
 }
 cumfreq <- matrix((sapply(1:length(freqdist), calc.cumfreq,
   freq = freqdist, dist = gowdist))*(1/n), ncol = length(freqdist))
 dimnames(cumfreq) <- list(seq(1, m, by = 1), freqdist)

 #RETURN
 out <- list(call = match.call(), in.hull = test.result, gowers.dist =
   t(gowdist), geom.var = gv.x, sum.stat = summary, cum.freq = cumfreq)
 class(out) <- "whatif"
 if (identical(TRUE, any(samp.range == 0)))  {
   warning("Range of at least one variable equals zero")
 }
 return(invisible(out))
}
