\name{whatif}
\alias{whatif}
\title{Counterfactual Evaluation}
\description{
  Implements the methods described in King and Zeng (2006a, 2006b) for
  evaluating counterfactuals.  
}
\usage{
whatif(data, cfact, range = NULL, freq = NULL, nearby = NULL, miss = "list", 
form = NULL, ...)
}
\arguments{
  \item{data}{An object or a character string.  If an
    object, either Zelig output (an object of class "z.out") or a
    n-by-k numeric matrix or data frame of
    observed covariate data, with n data points or units and k covariates.  
    These k covariates should include transformations
    of covariates and interaction terms, which may be obtained by
    passing model output (e.g., from a call to \code{lm}) to 
    \code{model.matrix} and excluding the
    first column of the resulting matrix (the intercept) if an intercept
    was fit. The program will coerce data frames to their
    internal numeric values.  If a string, either the
    complete path (including file name) of the file containing the
    data or the path relative to your working directory.  
    Missing data is allowed and will be dealt with via the
    argument \code{missing}.  It should be flagged using the
    standard representation for missing data, NA.}
  \item{cfact}{The counterfactual(s).  An object or a string.  If an
    object, a m-by-k numeric matrix or data frame of counterfactuals 
    with m counterfactuals and k covariates.  Data frames will again
    be coerced to their internal numeric values.  If a 
    string, either the complete path (including file name) of the file
    containing the counterfactuals or the path relative to your working directory.  
    This file should be a white space delimited text file without a header.
    In both cases, counterfactuals are assumed to be fully observed.}
  \item{range}{An optional numeric vector of length k (the number
    of covariates).  Each element represents the range of the
    corresponding covariate for use in calculating Gower distances.
    Use this argument when covariate data do not represent the population of interest,
    such as selection by stratification or experimental manipulation.
    By default, the range of each covariate is calculated
    from the data (the difference of its maximum and minimum values in
    the sample), which is appropriate when a simple random sampling
    design was used.  To supply your own range for the kth covariate,
    set the kth element of the vector equal to the desired range
    and all other elements equal to NA. 
    Default is NULL.}
  \item{freq}{An optional numeric vector of indeterminate length,
    including a set of Gower distances, for use in calculating
    cumulative frequency distributions for the distances of the data
    points from each counterfactual.  For each such Gower distance and
    counterfactual, the cumulative frequency is the fraction of observed
    covariate data points with Gower distance to the counterfactual less
    than or equal to the supplied Gower distance value.  By default,
    frequencies are calculated for the sequence of Gower distances from
    0 to 1 in increments of 0.05.  Default is NULL.}
  \item{nearby}{An optional scalar; the cutoff Gower distance value determining which observed
    data points are considered to be nearby the counterfactuals.
    Used to calculate the summary statistic returned by the function:
    the fraction of the observed data nearby each counterfactual.
    By default, the geometric variance of the covariate data is used.
    E.g., setting \code{nearby} equal to 0.10 will identify the proportion of
    data points within 0.10 of a counterfactual.  Default is NULL.}
  \item{miss}{An optional string indicating the strategy for dealing
  with missing data in the observed covariate data set.
  \code{whatif} supports two possible missing data strategies:
  "list", list-wise deletion of missing cases; and "case",
  ignoring missing data case-by-case.  Note that if "case" is
  selected, cases with missing values are deleted listwise for the
  convex hull test but simply ignored in computing the Gower
  distances.  Default is "list".}
  \item{form}{An optional formula without a dependent variable that is
  of class ``formula'' and follows standard R conventions for
  formulas, e.g. ~ x1 + x2.  Used to drop, transform, or otherwise
  respecify combinations of the variables in both \code{data} and
  \code{cfact}.  Primarily intended for dropping the key causal
  variable when identifying common support, which is done by including
  all covariates, seperated by a "+", to the right of the tilda in
  \code{form} but not the key causal variable.  Both
  \code{data} and \code{cfact} must be data frames in order to use
  this parameter; otherwise an error is returned.  Default is NULL.}
}
\details{
  This function is the primary tool for evaluating your counterfactuals.  
  Specifically, it:
  \enumerate{
    \item Determines whether or not your counterfactuals are in the convex hull of the
      observed covariate data.  
    \item Calculates the distance of your counterfactuals from each of the n
      observed covariate data points.  The distance function used is Gower's 
      nonparametric measure.
    \item Calculates a summary statistic for each counterfactual based on 
      the above Gower distances:  the fraction of observed covariate data points with 
      Gower distances to your counterfactual less than the value you supply.  By
      default, this value is taken to be the geometric variance of the observed
      data.
    \item Calculates the cumulative frequency distribution of each counterfactual
      for the distances in (2) using Gower distances you supply.  By default, Gower
     distances from 0 to 1 in increments of 0.05 are used.
  }
}
\value{
  An object of class "whatif", which is a list consisting of the following 
  six elements:
  \item{call}{The original call to \code{whatif}.}
  \item{in.hull}{A logical vector of length m, where m represents the number
    of counterfactuals.  Each element of the vector is a Boolean indicating
    whether or not the corresponding counterfactual is TRUE if the corresponding
    counterfactual is in the convex hull and FALSE otherwise.}
  \item{gowers.dist}{A m-by-n numeric matrix, where m is the number of counterfactuals
    and n is the number of data points (units).  The [i, j]th
    entry of the matrix contains the Gower distance between the ith counterfactual
    and the jth data point.}
  \item{geom.var}{A scalar.  The geometric variance of the observed covariate
    data.}
  \item{sum.stat}{A numeric vector of length m (the
    number of counterfactuals) containing the summary statistic for the
    corresponding counterfactual.  This summary statistic is the fraction of data
    points with Gower distances to the counterfactual less than the
    argument \code{nearby}, which is the geometric variance of the
    covariates by default.}
  \item{cum.freq}{A numeric matrix.  By default, the matrix has
    dimension m-by-21, where m is the number of
    counterfactuals; however, if you supplied your own frequencies via
    the argument \code{freq}, the matrix has dimension m-by-f,
    where f is the length of \code{freq}.  Each column of the
    matrix contains the cumulative frequency distribution for the
    corresponding counterfactual calculated using either the default
    set of Gower distance values or the set you supplied (see the
    discussion under the argument \code{freq}).  Each entry [i, j]
    of the matrix is thus the fraction of data points with Gower
    distances to the ith counterfactual less than or equal to the
    value represented by the jth row.  The column names contain these
    values.}
}
\references{King, Gary and Langche Zeng.  2006a.  "The Dangers of 
  Extreme Counterfactuals."  \emph{Political Analysis,} forthcoming.
  Preprint available from \url{http://gking.harvard.edu}.

  King, Gary and Langche Zeng.  2006b.  "When Can History Be Our Guide?
  The Pitfalls of Counterfactual Inference."  \emph{International Studies Quarterly,}
  forthcoming.  Preprint available from \url{http://gking.harvard.edu}.}
\author{Stoll, Heather \email{hstoll@polsci.ucsb.edu}, King, Gary
  \email{king@harvard.edu} and Zeng, Langche \email{zeng@ucsd.edu}}
\note{This function requires the \pkg{lpSolve} package.}
\seealso{
  \code{\link{plot.whatif}},
  \code{\link{summary.whatif}},
  \code{\link{print.whatif}},
  \code{\link{print.summary.whatif}}
}
\examples{
##  Create example data sets and counterfactuals
my.cfact <- matrix(rnorm(3*5), ncol = 5)
my.data <- matrix(rnorm(100*5), ncol = 5)

##  Evaluate counterfactuals
my.result <- whatif(data = my.data, cfact = my.cfact)

##  Evaluate counterfactuals and supply own gower distances for 
##  cumulative frequency distributions
my.result <- whatif(cfact = my.cfact, data = my.data, freq = c(0, .25, .5, 1, 1.25, 1.5))
}
\keyword{htest}
\keyword{models}
\keyword{regression}
